@@ -1,9 +1,11 @@
+#!/usr/bin/env python
 """
 Note that this module only works on trains from the Deutsche Bahn
 Note that this module only works while connected to the on board network "WIFI@DB" or "WIFIonICE"
 """
 import requests
 from datetime import datetime, timedelta
+from math import pi, sin, cos, sqrt, atan, radians
 
 class NetworkException(Exception):
     """Exception raised when a request fails to fetch data from the api
@@ -62,7 +64,7 @@ class WrongApiException(Exception):
             return self.message
 
 
-def convert_time_to_string(timedelta_obj, locale="en"):
+def convert_time_to_string(timedelta_obj, locale="en", no_seconds=False):
     """Converts a timedelta object into a readable string.
        timedelta: timedelta object
        locale: The language code for the returned string
@@ -109,7 +111,7 @@ def convert_time_to_string(timedelta_obj, locale="en"):
             string_time += f' {minutes} {strings_one[2]}'
         else:
             string_time += f' {minutes} {strings_many[2]}'
-    if not seconds == 0:
+    if not seconds == 0 and not no_seconds:
         if seconds == 1:
             string_time += f' {seconds} {strings_one[3]}'
         else:
@@ -121,6 +123,25 @@ def cut_timestamp(seconds):
     """
     return int(str(seconds)[:10])
 
+def calc_distance(position_start, position_end):
+    """Calculates the distance beween two positions in format (lat, lon)
+    """
+    f = 1/298.257223563
+    a = 6378173
+    F = radians((position_start[0] + position_end[0])/2.0)
+    G = radians((position_start[0] - position_end[0])/2.0)
+    l = radians((position_start[1] - position_end[1])/2.0)
+    S = sin(G)**2 * cos(l)**2 + cos(F)**2 * sin(l)**2
+    C = cos(G)**2 * cos(l)**2 + sin(F)**2 * sin(l)**2
+    w = atan(sqrt(S/C))
+    if float(w) == 0.0:
+        return 0.0
+    D = 2 * w * a
+    T = sqrt(S*C)/w
+    H_1 = (3*T-1)/(2*C)
+    H_2 = (3*T+1)/(2*S)
+    return D * (1 + f * H_1 * sin(F)**2 * cos(G)**2 - f * H_2 * cos(F)**2 * sin(G)**2)
+
 def request_json(url):
     """Requests data from 'url' and parses it as a dictionary
     """
@@ -361,7 +382,7 @@ def get_time_until_next_arrival(trip_call=None):
         trip = trip_call
     else:
         trip = get_trip()
-    return get_time_until_arrival(evaNr=get_next_station_eva_number(), trip_call=trip)
+    return get_time_until_arrival(evaNr=get_next_station_eva_number(trip_call=trip_call), trip_call=trip)
 
 def get_departure_time(station_name=None, evaNr=None, trip_call=None):
     """Gets the departure time at a specific station.
@@ -406,7 +427,7 @@ def get_time_until_next_departure(trip_call=None):
         trip = trip_call
     else:
         trip = get_trip()
-    return get_time_until_departure(evaNr=get_next_station_eva_number(), trip_call=trip)
+    return get_time_until_departure(evaNr=get_next_station_eva_number(trip_call=trip_call), trip_call=trip)
 
 def get_track(station_name=None, evaNr=None, trip_call=None):
     """Gets the track for a specific station
@@ -432,7 +453,7 @@ def get_track(station_name=None, evaNr=None, trip_call=None):
 def get_next_track(trip_call=None):
     """Gets the track of the next stop
     """
-    return get_track(evaNr=get_next_station_eva_number(), trip_call=trip_call)
+    return get_track(evaNr=get_next_station_eva_number(trip_call=trip_call), trip_call=trip_call)
 
 def get_delay(trip_call=None):
     """Gets the delay in minutes.
@@ -441,7 +462,7 @@ def get_delay(trip_call=None):
         trip = trip_call
     else:
         trip = get_trip()
-    evaNr = get_next_station_eva_number(trip)
+    evaNr = get_next_station_eva_number(trip_call=trip_call)
     for stop in trip['trip']['stops']:
         if stop['station']['evaNr'] == evaNr:
             if stop['timetable']['arrivalDelay'] == '':
@@ -450,7 +471,7 @@ def get_delay(trip_call=None):
                 return int(stop['timetable']['arrivalDelay'])
     raise NotAvailableException()
 
-def get_delay_reasons(trip_call=None):
+def get_all_delay_reasons(trip_call=None):
     """Gets all reasons for delays
     """
     if trip_call != None:
@@ -469,6 +490,23 @@ def get_delay_reasons(trip_call=None):
         return reasons
     raise NoneDataException() # Train is on time
 
+def get_delay_reasons(trip_call=None):
+    """Gets the current delay reasons
+    """
+    if trip_call != None:
+        trip = trip_call
+    else:
+        trip = get_trip()
+    reasons = []
+    
+    for reason in get_delay_reasons_last_station(trip_call=trip_call), get_delay_reasons_for_station(evaNr=get_next_station_eva_number(trip_call=trip_call), trip_call=trip_call):
+        if reason != []:
+            for r in reason:
+                reasons.append(r)
+    if len(reasons) != 0:
+        return list(dict.fromkeys(reasons))
+    raise NoneDataException() # Train is on time
+
 def get_delay_reasons_for_station(station_name=None, evaNr=None, trip_call=None):
     """Gets the delay reasons for a specific station
     """
@@ -497,7 +535,7 @@ def get_delay_reasons_for_station(station_name=None, evaNr=None, trip_call=None)
 def get_delay_reasons_last_station(trip_call=None):
     """Gets the reasons for the current delay
     """
-    return get_delay_reasons_for_station(evaNr=get_last_station_eva_number())
+    return get_delay_reasons_for_station(evaNr=get_last_station_eva_number(trip_call=trip_call))
 
 def get_delay_status(trip_call=None):
     """Gets the status of whether the train is delayed or not.
@@ -510,4 +548,33 @@ def get_delay_status(trip_call=None):
 def get_is_delayed(trip_call=None):
     """Alias for get_delay_status().
     """
-    return get_delay_status(trip_call=trip_call)
\ No newline at end of file
+    return get_delay_status(trip_call=trip_call)
+
+def get_station_position(station_name=None, evaNr=None, trip_call=None):
+    """Gets the position of a specific station
+    """
+    if trip_call == None:
+        trip_call = get_trip()
+    if station_name != None:
+        key = 'name'
+        value = station_name
+    elif evaNr != None:
+        key = 'evaNr'
+        value = evaNr
+    else:
+        raise TypeError('Missing at least one argument: station_name, evaNr')
+    
+    for stop in trip_call['trip']['stops']:
+        if stop['station'][key] == value:
+            return (stop['station']['geocoordinates']['latitude'], stop['station']['geocoordinates']['longitude'])
+
+def get_station_distance(station_name=None, evaNr=None, status_call=None, trip_call=None):
+    """Calculates the distance to a specific station and returns it in meters
+    """
+    if trip_call == None:
+        trip_call = get_trip()
+    if status_call == None:
+        status_call = get_status()
+    else:
+        raise TypeError('Missing at least one argument: station_name, evaNr')
+    return calc_distance(get_position(status_call=status_call), get_station_position(station_name=station_name, evaNr=evaNr, trip_call=trip_call))
